@@ -54,7 +54,7 @@ class KDEVPLATFORMLANGUAGE_EXPORT CodeCompletionWorker : public QObject
   Q_OBJECT
 
   public:
-    CodeCompletionWorker(QObject* parent);
+    CodeCompletionWorker(CodeCompletionModel* model);
     virtual ~CodeCompletionWorker();
 
     virtual void abortCurrentCompletion();
@@ -75,18 +75,24 @@ class KDEVPLATFORMLANGUAGE_EXPORT CodeCompletionWorker : public QObject
     
   protected:
     
-    virtual void computeCompletions(DUContextPointer context, const KTextEditor::Cursor& position, KTextEditor::View* view, const KTextEditor::Range& contextRange, const QString& contextText);
+    virtual void computeCompletions(DUContextPointer context, const KTextEditor::Cursor& position, QString followingText, const KTextEditor::Range& contextRange, const QString& contextText);
+    ///This can be overriden to compute an own grouping in the completion-list.
+    ///The default implementation groups items in a way that improves the efficiency of the completion-model, thus the default-implementation should be preferred.
     virtual QList<KSharedPtr<CompletionTreeElement> > computeGroups(QList<CompletionTreeItemPointer> items, KSharedPtr<CodeCompletionContext> completionContext);
     ///If you don't need to reimplement computeCompletions, you can implement only this.
     virtual KDevelop::CodeCompletionContext* createCompletionContext(KDevelop::DUContextPointer context, const QString &contextText, const QString &followingText, const CursorInRevision &position) const;
 
+    ///Override this to change the text-range which is used as context-information for the completion context
+    ///The foreground-lock and a DUChain read lock are held when this is called
+    virtual void updateContextRange(KTextEditor::Range& contextRange, KTextEditor::View* view, DUContextPointer context) const;
+    
     ///Can be used to retrieve and set the aborting flag(Enabling it is equivalent to caling abortCompletion())
     ///Is always reset from within computeCompletions
     bool& aborting();
     
     ///Emits foundDeclarations() with an empty list. Always call this when you abort the process of computing completions
     void failed();
-    
+
   public Q_SLOTS:
     ///Connection from the foreground thread within CodeCompletionModel
     void computeCompletions(KDevelop::DUContextPointer context, const KTextEditor::Cursor& position, KTextEditor::View* view);
@@ -99,6 +105,7 @@ class KDEVPLATFORMLANGUAGE_EXPORT CodeCompletionWorker : public QObject
     QMutex* m_mutex;
     bool m_abort;
     bool m_fullCompletion;
+    KDevelop::CodeCompletionModel* m_model;
 };
 
 }
