@@ -16,8 +16,8 @@
    Boston, MA 02110-1301, USA.
 */
 
-#ifndef APPENDEDLIST_H
-#define APPENDEDLIST_H
+#ifndef APPENDEDLIST_STATIC_H
+#define APPENDEDLIST_STATIC_H
 
 #include <util/kdevvarlengtharray.h>
 #include "referencecounting.h"
@@ -44,9 +44,9 @@ namespace KDevelop {
 // Foreach macro that takes a container and a function-name, and will iterate through the vector returned by that function, using the length returned by the function-name with "Size" appended.
 //This might be a little slow
 #define FOREACH_FUNCTION_STATIC(item, container) \
-      for(uint a = 0, mustDo = 1; a < container ## Size(); ++a) \
-        if((mustDo == 0 || mustDo == 1) && (mustDo = 2)) \
-          for(item(container()[a]); mustDo; mustDo = 0)
+      for(uint a__ = 0, mustDo__ = 1; a__ < container ## Size(); ++a__) \
+        if((mustDo__ == 0 || mustDo__ == 1) && (mustDo__ = 2)) \
+          for(item(container()[a__]); mustDo__; mustDo__ = 0)
 
 #define START_APPENDED_LISTS_STATIC(selftype) typedef selftype SelfType;
 
