@@ -24,7 +24,6 @@
 #include <QtCore/QMap>
 //#include <QtCore/qcontainerfwd.h>
 
-#include "../editor/documentrangeobject.h"
 #include "identifier.h"
 #include "indexedstring.h"
 #include "types/abstracttype.h"
@@ -34,10 +33,6 @@
 
 class QByteArray;
 
-namespace KTextEditor {
-  class SmartRange;
-}
-
 namespace KDevelop
 {
 
@@ -87,7 +82,7 @@ public:
    * \param range range of the alias declaration's identifier
    * \param parentContext context in which this declaration occurred
    * */
-  Declaration(const SimpleRange& range, DUContext* parentContext);
+  Declaration(const RangeInRevision& range, DUContext* parentContext);
   ///Copy-constructor for cloning
   Declaration(const Declaration& rhs);
   /// Destructor
@@ -132,7 +127,33 @@ public:
   bool isFinal() const;
   /// Set whether the declaration is final. \param final true if the declaration is final, otherwise false.
   void setFinal(bool final);
-
+  
+  /**
+   * Changes whether this declaration must be direct in all cases or not. Needed for some languages, i.e. Python.
+   * "false" is the default.
+   * \param direct true to force direct, false otherwise.
+   * */
+  void setAlwaysForceDirect(bool direct);
+  /// Access whether this declaration must always be direct.
+  bool alwaysForceDirect() const;
+  
+  /**
+   * Changes whether this declaration is "implicitly created", i.e. not declared in the class context, but written somewhere else outside
+   * \param _auto true for implicit, false for default behaviour
+   * */
+  void setAutoDeclaration(bool _auto);
+  /// Access whether this declaration is implicitly created or not.
+  bool isAutoDeclaration() const;
+  
+  /**
+   * Changes whether this declaration is "explicitly deleted", i.e. not implicitly declared or accessible.
+   *
+   * \param deleted true for deleted, false for default behaviour
+   * */
+  void setExplicitlyDeleted(bool deleted);
+  /// Access whether this declaration is implicitly created or not.
+  bool isExplicitlyDeleted() const;
+  
   /**
    * Retrieve the declaration which is specialized with the given \a specialization index as seen from \a topContext.
    *
@@ -335,23 +356,33 @@ public:
    */
   virtual QString toString() const;
 
-  ///@todo The following two are convenience-functions. Think whether they should stay here, or be moved out.
+  /**
+   * Returns a list of pairs:
+   * An url of a file, paired together with all use-ranges of this declaration in that file.
+   * The ranges are in the documents local revision (use DUChainUtils::transformFromRevision or usesCurrentRevision())
+   * The uses are unique, no 2 uses are returend that have the same range within the same file.
+   * 
+   * This is a non-trivial operation.
+   * */
+  QMap<IndexedString, QList<RangeInRevision> > uses() const;
+  
+  /**
+   * Determines whether the declaration has any uses or not.
+   * Cheaper than calling uses().
+   * */
+  bool hasUses() const;
 
   /**
    * Returns a list of pairs:
    * An url of a file, paired together with all use-ranges of this declaration in that file.
+   * The ranges are in the most current document revisions available.
    * The uses are unique, no 2 uses are returend that have the same range within the same file.
-   *
+   * 
+   * @warning This must be called only from within the foreground, or with the foreground lock locked.
    * This is a non-trivial operation.
    * */
-  QMap<IndexedString, QList<SimpleRange> > uses() const;
-
-  /**
-   * Collects the smart-ranges for all uses. Uses that do not have smart ranges are not represented
-   * in the result.
-   * */
-  QList<KTextEditor::SmartRange*> smartUses() const;
-
+  QMap<IndexedString, QList<SimpleRange> > usesCurrentRevision() const;
+  
   /**
     * This hash-value should differentiate between multiple different
     * declarations that have the same qualifiedIdentifier, but should have a different
@@ -368,7 +399,8 @@ public:
 
   /**
    * @see DeclarationId
-   * @param forceDirect When this is true, the DeclarationId is force to be direct, and can be resolved without a symbol-table and top-context
+   * @param forceDirect When this is true, the DeclarationId is force to be direct, and can be resolved without a symbol-table and top-context.
+   * The same goes for Declarations that have @c alwaysForceDirect() set to true.
    * */
   virtual DeclarationId id(bool forceDirect = false) const;
 
@@ -427,7 +459,7 @@ protected:
     * \param url document url in which this object is located.
     * \param range text range which this object covers.
     */
-  Declaration( DeclarationData & dd, const SimpleRange& range );
+  Declaration( DeclarationData & dd, const RangeInRevision& range );
 
   /**
    * Returns true if this declaration is being currently destroyed persistently,
