@@ -81,15 +81,17 @@ namespace KDevelop {
       
     Q_SIGNALS:
       void sizeHintChanged();
+      /// Emitted whenever the current navigation-context has changed
+      /// @param wasInitial whether the previous context was the initial context
+      /// @param isInitial whether the current context is the initial context
+      void contextChanged(bool wasInitial, bool isInitial);
     public slots:
       void navigateDeclaration(KDevelop::IndexedDeclaration decl);
     private slots:
       void anchorClicked(const QUrl&);
     protected:
-      
       virtual void wheelEvent(QWheelEvent* );
       void updateIdealSize() const;
-      virtual void keyPressEvent(QKeyEvent* event);
 
       void initBrowser(int height);
       void update();
