@@ -32,6 +32,8 @@
 #include "../classmemberdeclaration.h"
 #include "../abstractfunctiondeclaration.h"
 #include "../functiondefinition.h"
+#include <interfaces/iuicontroller.h>
+#include <codegen/coderepresentation.h>
 
 using namespace KDevelop;
 
@@ -232,6 +234,30 @@ void UsesCollector::startCollecting() {
         
         if(checker(file))
           collected.insert(file);
+        
+        {
+          QSet<ParsingEnvironmentFile*> filteredCollected;
+          QMap<IndexedString, bool> grepCache;
+          // Filter the collected files by performing a grep
+          foreach(ParsingEnvironmentFile* file, collected)
+          {
+            IndexedString url = file->url();
+            QMap< IndexedString, bool >::iterator grepCacheIt = grepCache.find(url);
+            if(grepCacheIt == grepCache.end())
+            {
+              CodeRepresentation::Ptr repr = KDevelop::createCodeRepresentation( url );
+              if(repr)
+              {
+                QVector<SimpleRange> found = repr->grep(decl->identifier().identifier().str());
+                grepCacheIt = grepCache.insert(url, !found.isEmpty());
+              }
+            }
+            if(grepCacheIt.value())
+              filteredCollected << file;
+          }
+          kDebug() << "Collected contexts for full re-parse, before filtering: " << collected.size() << " after filtering: " << filteredCollected.size();
+          collected = filteredCollected;
+        }
 
         ///We have all importers now. However since we can tell parse-jobs to also update all their importers, we only need to
         ///update the "root" top-contexts that open the whole set with their imports.
@@ -252,9 +278,9 @@ void UsesCollector::startCollecting() {
         emit maximumProgressSignal(rootFiles.size());
         maximumProgress(rootFiles.size());
 
-        //If we used the AllDeclarationsContextsAndUsesRecursive flag here, we would compute way too much. This we only
-        //set the minimum-features selectively on the files we encountered.
-        foreach(ParsingEnvironmentFile* file, visited)
+        //If we used the AllDeclarationsContextsAndUsesRecursive flag here, we would compute way too much. This way we only
+        //set the minimum-features selectively on the files we really require them on.
+        foreach(ParsingEnvironmentFile* file, collected)
           m_staticFeaturesManipulated.insert(file->url());
         m_staticFeaturesManipulated.insert(decl->url());
 
@@ -337,14 +363,22 @@ void UsesCollector::updateReady(KDevelop::IndexedString url, KDevelop::Reference
   if(!m_staticFeaturesManipulated.contains(url))
     return; //Not interesting
 
-  if(!(topContext->features() & TopDUContext::AllDeclarationsContextsAndUses) || topContext->parsingEnvironmentFile()->needsUpdate()) {
+  if(!(topContext->features() & TopDUContext::AllDeclarationsContextsAndUses)) {
       ///@todo With simplified environment-matching, the same file may have been imported multiple times,
       ///while only one of  those was updated. We have to check here whether this file is just such an import,
       ///or whether we work on with it.
       ///@todo We will lose files that were edited right after their update here.
-//       kDebug() << "context" << topContext->url().str() << "does not have the required features";
+      kWarning() << "WARNING: context" << topContext->url().str() << "does not have the required features!!";
+      ICore::self()->uiController()->showErrorMessage("Updating " + ICore::self()->projectController()->prettyFileName(topContext->url().toUrl(), KDevelop::IProjectController::FormatPlain) + " failed!", 5);
       return;
   }
+  
+  if(topContext->parsingEnvironmentFile()->needsUpdate()) {
+      kWarning() << "WARNING: context" << topContext->url().str() << "is not up to date!";
+      ICore::self()->uiController()->showErrorMessage(ICore::self()->projectController()->prettyFileName(topContext->url().toUrl(), KDevelop::IProjectController::FormatPlain) + " still needs an update!", 5);
+//       return;
+  }
+  
 
     IndexedTopDUContext indexed(topContext.data());
     if(m_checked.contains(indexed))
