@@ -53,10 +53,10 @@ private slots:
     void testCommit();
     void testBranching();
     void revHistory();
+    void testAnnotation();
     void cleanupTestCase();
 
 private:
-    KDevelop::TestCore* m_testCore;
     GitPlugin* m_plugin;
     void removeTempDirs();
 };
