@@ -63,7 +63,7 @@ bool KDevelop::removeUrl(const KDevelop::IProject* project, const KUrl& url, con
 
 bool KDevelop::createFile(const KUrl& file)
 {
-    if (KIO::NetAccess::exists( file, KIO::NetAccess::SourceSide, QApplication::activeWindow() )) {
+    if (KIO::NetAccess::exists( file, KIO::NetAccess::DestinationSide, QApplication::activeWindow() )) {
         KMessageBox::error( QApplication::activeWindow(),
                             i18n( "The file <i>%1</i> exists already.", file.pathOrUrl() ) );
         return false;
