@@ -75,6 +75,9 @@ Boston, MA 02110-1301, USA.
 #include "sessioncontroller.h"
 #include "session.h"
 #include <QApplication>
+#include <vcs/models/projectchangesmodel.h>
+#include <vcs/widgets/vcsdiffpatchsources.h>
+#include <vcs/widgets/vcscommitdialog.h>
 
 namespace KDevelop
 {
@@ -100,6 +103,7 @@ public:
     ProjectBuildSetModel* buildset;
     bool m_foundProjectFile; //Temporary flag used while searching the hierarchy for a project file
     bool m_cleaningUp; //Temporary flag enabled while destroying the project-controller
+    QPointer<ProjectChangesModel> m_changesModel;
 
     ProjectControllerPrivate( ProjectController* p )
         : m_core(0), model(0), selectionModel(0), dialog(0), m_configuringProject(0), q(p), m_foundProjectFile(false), m_cleaningUp(false)
@@ -265,7 +269,8 @@ public:
         emit q->projectAboutToBeOpened( project );
         if ( !project->open( url ) )
         {
-            delete project;
+            q->abortOpeningProject(project);
+            project->deleteLater();
             return;
         }
 
@@ -356,7 +361,7 @@ KUrl ProjectDialogProvider::askProjectConfigLocation(bool fetch, const KUrl& sta
             no.setToolTip(i18n("Continue to open the project but use the existing project configuration."));
             no.setIcon(KIcon());
             KGuiItem cancel = KStandardGuiItem::cancel();
-            cancel.setToolTip(i18n("Cancel and don't open the project."));
+            cancel.setToolTip(i18n("Cancel and do not open the project."));
             int ret = KMessageBox::questionYesNoCancel(qApp->activeWindow(),
                 i18n("There already exists a project configuration file at %1.\n"
                      "Do you want to override it or open the existing file?", projectFileUrl.pathOrUrl()),
@@ -386,6 +391,7 @@ KUrl ProjectDialogProvider::askProjectConfigLocation(bool fetch, const KUrl& sta
         bool ok = writeNewProjectFile( KSharedConfig::openConfig( tmp.fileName(), KConfig::SimpleConfig ),
                         dlg.projectName(),
                         dlg.projectManager() );
+        tmp.close();
         if (!ok)
             return KUrl();
 
@@ -422,12 +428,12 @@ ProjectController::ProjectController( Core* core )
     d->m_core = core;
     d->model = new ProjectModel();
     d->buildset = new ProjectBuildSetModel( this );
-    connect( this, SIGNAL( projectOpened( KDevelop::IProject* ) ),
-             d->buildset, SLOT( loadFromProject( KDevelop::IProject* ) ) );
-    connect( this, SIGNAL( projectClosing( KDevelop::IProject* ) ),
-             d->buildset, SLOT( saveToProject( KDevelop::IProject* ) ) );
-    connect( this, SIGNAL( projectClosed( KDevelop::IProject* ) ),
-             d->buildset, SLOT( projectClosed( KDevelop::IProject* ) ) );
+    connect( this, SIGNAL(projectOpened(KDevelop::IProject*)),
+             d->buildset, SLOT(loadFromProject(KDevelop::IProject*)) );
+    connect( this, SIGNAL(projectClosing(KDevelop::IProject*)),
+             d->buildset, SLOT(saveToProject(KDevelop::IProject*)) );
+    connect( this, SIGNAL(projectClosed(KDevelop::IProject*)),
+             d->buildset, SLOT(projectClosed(KDevelop::IProject*)) );
 
     d->selectionModel = new QItemSelectionModel(d->model);
     if(!(Core::self()->setupFlags() & Core::NoUi)) setupActions();
@@ -451,40 +457,45 @@ void ProjectController::setupActions()
     action->setToolTip( i18n( "Open / Import Project" ) );
     action->setWhatsThis( i18n( "<b>Open / Import project</b><p>Open an existing KDevelop 4 project or import an existing Project into KDevelop 4. This entry allows to select a KDevelop4 project file or an existing directory to open it in KDevelop. When opening an existing directory that does not yet have a KDevelop4 project file, the file will be created.</p>" ) );
     action->setIcon(KIcon("project-open"));
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( openProject() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(openProject()) );
     
     d->m_fetchProject = action = ac->addAction( "project_fetch" );
     action->setText(i18n( "Fetch Project..." ) );
     action->setToolTip( i18n( "Fetch Project" ) );
     action->setWhatsThis( i18n( "<b>Fetch project</b><p>Guides the user through the project fetch and then imports it into KDevelop 4.</p>" ) );
 //     action->setIcon(KIcon("project-open"));
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( fetchProject() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(fetchProject()) );
 
 //    action = ac->addAction( "project_close" );
 //    action->setText( i18n( "C&lose Project" ) );
-//    connect( action, SIGNAL( triggered( bool ) ), SLOT( closeProject() ) );
+//    connect( action, SIGNAL(triggered(bool)), SLOT(closeProject()) );
 //    action->setToolTip( i18n( "Close project" ) );
 //    action->setWhatsThis( i18n( "<b>Close project</b><p>Closes the current project." ) );
 //    action->setEnabled( false );
 
     d->m_closeProject = action = ac->addAction( "project_close" );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( closeSelectedProjects() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(closeSelectedProjects()) );
     action->setText( i18n( "Close Project(s)" ) );
     action->setIcon( KIcon( "project-development-close" ) );
     action->setToolTip( i18n( "Closes all currently selected projects" ) );
     action->setEnabled( false );
 
     d->m_openConfig = action = ac->addAction( "project_open_config" );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( openProjectConfig() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(openProjectConfig()) );
     action->setText( i18n( "Open Configuration..." ) );
     action->setIcon( KIcon("configure") );
     action->setEnabled( false );
+    
+    action = ac->addAction( "commit_current_project" );
+    connect( action, SIGNAL(triggered(bool)), SLOT(commitCurrentProject()) );
+    action->setText( i18n( "Commit Current Project..." ) );
+    action->setIcon( KIcon("svn-commit") );
 
     KSharedConfig * config = KGlobal::config().data();
 //     KConfigGroup group = config->group( "General Options" );
 
     d->m_recentAction = new KRecentFilesAction( this );
-    connect( d->m_recentAction, SIGNAL(urlSelected(const KUrl&)), SLOT(
+    connect( d->m_recentAction, SIGNAL(urlSelected(KUrl)), SLOT(
                             openProject( const KUrl& ) ));
     ac->addAction( "project_open_recent", d->m_recentAction );
     d->m_recentAction->setText( i18n( "Open Recent" ) );
@@ -524,13 +535,18 @@ void ProjectController::initialize()
     KConfigGroup group = config->group( "General Options" );
     KUrl::List openProjects = group.readEntry( "Open Projects", QStringList() );
 
-    foreach (const KUrl& url, openProjects)
-        openProject(url);
-
+    QMetaObject::invokeMethod(this, "openProjects", Qt::QueuedConnection, Q_ARG(KUrl::List, openProjects));
+    
     connect( Core::self()->selectionController(), SIGNAL(selectionChanged(KDevelop::Context*)),
              SLOT(updateActionStates(KDevelop::Context*)) );
 }
 
+void ProjectController::openProjects(const KUrl::List& projects)
+{
+    foreach (const KUrl& url, projects)
+        openProject(url);
+}
+
 void ProjectController::loadSettings( bool projectIsLoaded )
 {
     Q_UNUSED(projectIsLoaded)
@@ -789,11 +805,6 @@ void ProjectController::closeProject(IProject* proj_)
     // loading might have failed
     d->m_currentlyOpening.removeAll(proj_->projectFileUrl());
 
-    if(d->m_projects.indexOf(proj_) == -1)
-    {
-        return;
-    }
-
     Project* proj = dynamic_cast<KDevelop::Project*>( proj_ );
     if( !proj ) 
     {
@@ -813,14 +824,20 @@ void ProjectController::closeProject(IProject* proj_)
     {
         initializePluginCleanup(proj);
     }
-    
+
     if(!d->m_cleaningUp)
         d->saveListOfOpenedProjects();
-    
+
     emit projectClosed(proj);
     return;
 }
 
+void ProjectController::abortOpeningProject(IProject* proj)
+{
+    d->m_currentlyOpening.removeAll(proj->projectFileUrl());
+    emit projectOpeningAborted(proj);
+}
+
 ProjectModel* ProjectController::projectModel()
 {
     return d->model;
@@ -907,7 +924,7 @@ QString ProjectController::prettyFilePath(KUrl url, FormattingOptions format) co
         } else {
             prefixText = project->name() + '/';
         }
-        QString relativePath = project->relativeUrl(url.upUrl()).path(KUrl::AddTrailingSlash);;
+        QString relativePath = project->relativeUrl(url.upUrl()).path(KUrl::AddTrailingSlash);
         if(relativePath.startsWith("./"))
             relativePath = relativePath.mid(2);
         prefixText += relativePath;
@@ -917,6 +934,16 @@ QString ProjectController::prettyFilePath(KUrl url, FormattingOptions format) co
 
 QString ProjectController::prettyFileName(KUrl url, FormattingOptions format) const
 {
+    IProject* project = Core::self()->projectController()->findProjectForUrl(url);
+    if(project && project->folder().equals(url, KUrl::CompareWithoutTrailingSlash))
+    {
+        if (format == FormatHtml) {
+            return "<i>" +  project->name() + "</i>";
+        } else {
+            return project->name();
+        }
+    }
+    
     QString prefixText = prettyFilePath( url, format );
     if (format == FormatHtml) {
         return prefixText + "<b>" + url.fileName() + "</b>";
@@ -942,6 +969,44 @@ ProjectBuildSetModel* ProjectController::buildSetModel()
     return d->buildset;
 }
 
+ProjectChangesModel* ProjectController::changesModel()
+{
+    if(!d->m_changesModel)
+        d->m_changesModel=new ProjectChangesModel(this);
+    
+    return d->m_changesModel;
+}
+
+void ProjectController::commitCurrentProject()
+{
+    IDocument* doc=ICore::self()->documentController()->activeDocument();
+    if(!doc)
+        return;
+    
+    KUrl url=doc->url();
+    IProject* project = ICore::self()->projectController()->findProjectForUrl(url);
+    
+    if(project && project->versionControlPlugin()) {
+        KUrl baseUrl=project->projectItem()->url();
+        IPlugin* plugin = project->versionControlPlugin();
+        IBasicVersionControl* vcs=plugin->extension<IBasicVersionControl>();
+        
+        if(vcs) {
+            ICore::self()->documentController()->saveAllDocuments(KDevelop::IDocument::Silent);
+
+            VCSCommitDiffPatchSource* patchSource = new VCSCommitDiffPatchSource(new VCSStandardDiffUpdater(vcs, baseUrl), baseUrl, vcs);
+
+            bool ret = showVcsDiff(patchSource);
+
+            if(!ret) {
+                VcsCommitDialog *commitDialog = new VcsCommitDialog(patchSource);
+                commitDialog->setCommitCandidates(patchSource->infos());
+                commitDialog->exec();
+            }
+        }
+    }
+}
+
 }
 
 #include "projectcontroller.moc"
