@@ -59,6 +59,9 @@
 #include <interfaces/icompletionsettings.h>
 
 #include <kdeversion.h>
+#include <interfaces/iprojectcontroller.h>
+#include <interfaces/iproject.h>
+#include <project/projectutils.h>
 
 namespace KDevelop {
 
@@ -96,7 +99,7 @@ struct TextDocumentPrivate {
         Q_UNUSED(document);
         m_textDocument->notifyContentChanged();
     }
-
+    
     void populateContextMenu( KTextEditor::View* v, QMenu* menu )
     {
         if (m_addedContextMenu) {
@@ -106,12 +109,24 @@ struct TextDocumentPrivate {
             delete m_addedContextMenu;
         }
 
+        m_addedContextMenu = new QMenu();
+
         Context* c = new EditorContext( v, v->cursorPosition() );
         QList<ContextMenuExtension> extensions = Core::self()->pluginController()->queryPluginsForContextMenuExtensions( c );
-        menu->addSeparator();
-
-        m_addedContextMenu = new QMenu();
+        
         ContextMenuExtension::populateMenu(m_addedContextMenu, extensions);
+        
+        {
+            KUrl url = v->document()->url();
+            IProject* project = Core::self()->projectController()->findProjectForUrl( url );
+            if(project)
+            {
+                QList< ProjectBaseItem* > items = project->itemsForUrl( url );
+                if(!items.isEmpty())
+                    populateParentItemsMenu( items.front(), m_addedContextMenu );
+            }
+        }
+        
         foreach ( QAction* action, m_addedContextMenu->actions() ) {
             menu->addAction(action);
         }
@@ -280,8 +295,8 @@ QWidget *TextDocument::createViewWidget(QWidget *parent)
         if (iface)
         {
             iface->setModifiedOnDiskWarning(true);
-            connect(d->document, SIGNAL(modifiedOnDisk(KTextEditor::Document*, bool,KTextEditor::ModificationInterface::ModifiedOnDiskReason)),
-                this, SLOT(modifiedOnDisk(KTextEditor::Document*, bool,KTextEditor::ModificationInterface::ModifiedOnDiskReason)));
+            connect(d->document, SIGNAL(modifiedOnDisk(KTextEditor::Document*,bool,KTextEditor::ModificationInterface::ModifiedOnDiskReason)),
+                this, SLOT(modifiedOnDisk(KTextEditor::Document*,bool,KTextEditor::ModificationInterface::ModifiedOnDiskReason)));
         }
 
         notifyTextDocumentCreated();
@@ -293,16 +308,19 @@ QWidget *TextDocument::createViewWidget(QWidget *parent)
         connect(view, SIGNAL(contextMenuAboutToShow(KTextEditor::View*,QMenu*)), this, SLOT(populateContextMenu(KTextEditor::View*,QMenu*)));
 
         //in KDE >= 4.4 we can use KXMLGuiClient::replaceXMLFile to provide
-        //katepart with out own restructured UI configuration
-        QStringList katePartUIs = KGlobal::mainComponent().dirs()->findAllResources("data", "kdevelop/katepartui.rc");
-        const QString katePartUI = katePartUIs.last();
-        const QString katePartLocalUI = KStandardDirs::locateLocal("data", "kdevelop/katepartui.rc");
-        if (!QFile::exists(katePartLocalUI)) {
-            // prevent warning:
-            // kdevelop/kdeui (kdelibs): No such XML file ".../.kde/share/apps/kdevelop/katepartui.rc"
-            QFile::copy(katePartUI, katePartLocalUI);
+        //katepart with our own restructured UI configuration
+        const QString uiFile = KGlobal::mainComponent().componentName() + "/katepartui.rc";
+        QStringList katePartUIs = KGlobal::mainComponent().dirs()->findAllResources("data", uiFile);
+        if (!katePartUIs.isEmpty()) {
+            const QString katePartUI = katePartUIs.last();
+            const QString katePartLocalUI = KStandardDirs::locateLocal("data", uiFile);
+            if (!QFile::exists(katePartLocalUI)) {
+                // prevent warning:
+                // kdevelop/kdeui (kdelibs): No such XML file ".../.kde/share/apps/kdevelop/katepartui.rc"
+                QFile::copy(katePartUI, katePartLocalUI);
+            }
+            view->replaceXMLFile(katePartUI, katePartLocalUI);
         }
-        view->replaceXMLFile(katePartUI, katePartLocalUI);
     }
 
     if (KTextEditor::CodeCompletionInterface* cc = dynamic_cast<KTextEditor::CodeCompletionInterface*>(view))
@@ -377,7 +395,7 @@ bool TextDocument::save(DocumentSaveMode mode)
     }
 
     KUrl urlBeforeSave = d->document->url();
-    if (d->document->save())
+    if (d->document->documentSave())
     {
         if (d->document->url() != urlBeforeSave)
             notifyUrlChanged();
@@ -655,14 +673,14 @@ void KDevelop::TextEditorWidget::setEditorView(KTextEditor::View* view)
 {
     if (d->view)
     {
-        disconnect(view, SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)));
+        disconnect(view, SIGNAL(cursorPositionChanged(KTextEditor::View*,KTextEditor::Cursor)));
         removeChildClient(view);
     }
 
     d->view = view;
     insertChildClient(d->view);
-    connect(view, SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), 
-            this, SLOT(viewStatusChanged(KTextEditor::View*, const KTextEditor::Cursor&)));
+    connect(view, SIGNAL(cursorPositionChanged(KTextEditor::View*,KTextEditor::Cursor)), 
+            this, SLOT(viewStatusChanged(KTextEditor::View*,KTextEditor::Cursor)));
     
     viewStatusChanged(view, view->cursorPosition());
 
