@@ -373,7 +373,7 @@ IdealDockWidget::IdealDockWidget(IdealController *controller, Sublime::MainWindo
     if (closeButton) {
     disconnect(closeButton, SIGNAL(clicked()), 0, 0);
 
-    connect(closeButton, SIGNAL(clicked(bool)), this, SLOT(slotRemove()));
+    connect(closeButton, SIGNAL(clicked(bool)), SIGNAL(close()));
     }
 
     setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable);
@@ -620,6 +620,7 @@ void IdealController::addView(Qt::DockWidgetArea area, View* view)
 
         m_docks->addAction(action);
         bar->show();
+        connect(dock, SIGNAL(close()), action, SLOT(toggle()));
     }
 
     connect(dock, SIGNAL(dockLocationChanged(Qt::DockWidgetArea)), this, SLOT(dockLocationChanged(Qt::DockWidgetArea)));
@@ -740,6 +741,15 @@ void IdealController::slotDockBarContextMenuRequested(QPoint position)
 
 void IdealController::raiseView(View* view, RaiseMode mode)
 {
+    /// @todo GroupWithOtherViews is disabled for now by forcing "mode = HideOtherViews".
+    ///       for the release of KDevelop 4.3.
+    ///       Reason: Inherent bugs which need significant changes to be fixed.
+    ///       Example: Open two equal toolviews (for example 2x konsole),
+    ///                activate one, switch area, switch back, -> Both are active instead of one.
+    ///       The problem is that views are identified purely by their factory-id, which is equal
+    ///       for toolviews of the same type.
+    mode = HideOtherViews;
+   
     QAction* action = m_view_to_action.value(view);
     Q_ASSERT(action);
 
