@@ -37,6 +37,7 @@
 #include "controller.h"
 #include "container.h"
 #include "ideal.h"
+#include "holdupdates.h"
 
 namespace Sublime {
 
@@ -48,6 +49,9 @@ MainWindow::MainWindow(Controller *controller, Qt::WindowFlags flags)
     loadGeometry(KGlobal::config()->group("Main Window"));
     d->areaSwitcher = new AreaTabWidget(menuBar());
     menuBar()->setCornerWidget(d->areaSwitcher, Qt::TopRightCorner);
+
+    // don't allow AllowTabbedDocks - that doesn't make sense for "ideal" UI
+    setDockOptions(QMainWindow::AnimatedDocks);
 }
 
 QWidget* MainWindow::customButtonForAreaSwitcher ( Area* /*area*/ )
@@ -115,13 +119,15 @@ MainWindow::~MainWindow()
     delete d;
 }
 
+void MainWindow::reconstructViews()
+{
+    d->reconstructViews();
+}
+
 void MainWindow::setArea(Area *area)
 {
     if (d->area)
         disconnect(d->area, 0, this, 0);
-    
-    bool wasEnabled = updatesEnabled();
-    setUpdatesEnabled(false);
 
     bool differentArea = (area != d->area);
     /* All views will be removed from dock area now.  However, this does
@@ -135,6 +141,7 @@ void MainWindow::setArea(Area *area)
     if (d->area)
         clearArea();
 
+    HoldUpdates hu(this);
     d->area = area;
     d->reconstruct();
     
@@ -147,24 +154,26 @@ void MainWindow::setArea(Area *area)
     emit areaChanged(area);
     d->ignoreDockShown = false;
     
-    loadSettings();
+    hu.stop();
+    // delay loading settings: we need to finish with area activation
+    // and only then load mainwindow/dockwidget settings
+    // this way dock sizes get properly restored
+    QMetaObject::invokeMethod(this, "loadSettings", Qt::QueuedConnection);
 
-    setUpdatesEnabled(wasEnabled);
-
-    connect(area, SIGNAL(viewAdded(Sublime::AreaIndex*, Sublime::View*)),
-        this, SLOT(viewAdded(Sublime::AreaIndex*, Sublime::View*)));
+    connect(area, SIGNAL(viewAdded(Sublime::AreaIndex*,Sublime::View*)),
+        this, SLOT(viewAdded(Sublime::AreaIndex*,Sublime::View*)));
     connect(area, SIGNAL(viewRemoved(Sublime::AreaIndex*,Sublime::View*)),
-        this, SLOT(viewRemovedInternal(Sublime::AreaIndex*, Sublime::View*)));
+        this, SLOT(viewRemovedInternal(Sublime::AreaIndex*,Sublime::View*)));
     connect(area, SIGNAL(requestToolViewRaise(Sublime::View*)),
         this, SLOT(raiseToolView(Sublime::View*)));
-    connect(area, SIGNAL(aboutToRemoveView(Sublime::AreaIndex*, Sublime::View*)),
-        this, SLOT(aboutToRemoveView(Sublime::AreaIndex*, Sublime::View*)));
-    connect(area, SIGNAL(toolViewAdded(Sublime::View*, Sublime::Position)),
-        this, SLOT(toolViewAdded(Sublime::View*, Sublime::Position)));
-    connect(area, SIGNAL(aboutToRemoveToolView(Sublime::View*, Sublime::Position)),
-        this, SLOT(aboutToRemoveToolView(Sublime::View*, Sublime::Position)));
-    connect(area, SIGNAL(toolViewMoved(Sublime::View*, Sublime::Position)),
-        this, SLOT(toolViewMoved(Sublime::View*, Sublime::Position)));
+    connect(area, SIGNAL(aboutToRemoveView(Sublime::AreaIndex*,Sublime::View*)),
+        this, SLOT(aboutToRemoveView(Sublime::AreaIndex*,Sublime::View*)));
+    connect(area, SIGNAL(toolViewAdded(Sublime::View*,Sublime::Position)),
+        this, SLOT(toolViewAdded(Sublime::View*,Sublime::Position)));
+    connect(area, SIGNAL(aboutToRemoveToolView(Sublime::View*,Sublime::Position)),
+        this, SLOT(aboutToRemoveToolView(Sublime::View*,Sublime::Position)));
+    connect(area, SIGNAL(toolViewMoved(Sublime::View*,Sublime::Position)),
+        this, SLOT(toolViewMoved(Sublime::View*,Sublime::Position)));
      connect(area, SIGNAL(changedWorkingSet(Sublime::Area*,QString,QString)),
         this, SLOT(setupAreaSelector()));
 }
@@ -254,7 +263,7 @@ void MainWindow::saveSettings()
     //debugToolBar visibility is stored separately to allow a area dependent default value
     foreach (KToolBar* toolbar, toolBars()) {
         if (toolbar->objectName() == "debugToolBar") {
-            cg.writeEntry("debugToolBarVisibility", toolbar->isVisible());
+            cg.writeEntry("debugToolBarVisibility", toolbar->isVisibleTo(this));
         }
     }
 
@@ -263,7 +272,9 @@ void MainWindow::saveSettings()
 
 void MainWindow::loadSettings()
 {
-    kDebug() << "loading settings for " << (area() ? area()->objectName() : "");
+    HoldUpdates hu(this);
+
+    kDebug(9504) << "loading settings for " << (area() ? area()->objectName() : "");
     QString group = "MainWindow";
     if (area())
         group += '_' + area()->objectName();
@@ -344,6 +355,8 @@ void MainWindow::loadSettings()
 
     cg.sync();
 
+    hu.stop();
+    
     emit settingsLoaded();
 }
 
@@ -399,7 +412,7 @@ void MainWindow::enableAreaSettingsSave()
 
 QWidget *MainWindow::statusBarLocation()
 {
-    return d->idealMainWidget->statusBarLocation();
+    return d->idealController->statusBarLocation();
 }
 
 void MainWindow::setAreaSwitcherCornerWidget(QWidget* widget)
@@ -417,7 +430,7 @@ void MainWindow::tabContextMenuRequested(View* , KMenu* )
     // do nothing
 }
 
-void MainWindow::tabToolTipRequested(View*, QPoint)
+void MainWindow::tabToolTipRequested(View*, Container*, int)
 {
     // do nothing
 }
@@ -441,6 +454,11 @@ View* MainWindow::viewForPosition(QPoint globalPos) const
     return 0;
 }
 
+void MainWindow::setBackgroundCentralWidget(QWidget* w)
+{
+    d->setBackgroundCentralWidget(w);
+}
+
 }
 
 #include "mainwindow.moc"
