@@ -42,6 +42,7 @@
 #include "controller.h"
 #include "mainwindow.h"
 #include "ideal.h"
+#include "holdupdates.h"
 #include <KToolBar>
 #include <KSelectAction>
 #include <ktoggleaction.h>
@@ -49,7 +50,7 @@
 namespace Sublime {
 
 MainWindowPrivate::MainWindowPrivate(MainWindow *w, Controller* controller)
-:controller(controller), area(0), activeView(0), activeToolView(0), centralWidget(0),
+:controller(controller), area(0), activeView(0), activeToolView(0), bgCentralWidget(0),
  ignoreDockShown(false), autoAreaSettingsSave(false), m_mainWindow(w)
 {
     KActionCollection *ac = m_mainWindow->actionCollection();
@@ -72,12 +73,6 @@ MainWindowPrivate::MainWindowPrivate(MainWindow *w, Controller* controller)
     connect(action, SIGNAL(toggled(bool)), SLOT(showBottomDock(bool)));
     ac->addAction("show_bottom_dock", action);
 
-    action = new KAction(i18n("Show Top Dock"), this);
-    action->setCheckable(true);
-    action->setShortcut(Qt::META | Qt::CTRL | Qt::Key_T);
-    connect(action, SIGNAL(toggled(bool)), SLOT(showTopDock(bool)));
-    ac->addAction("show_top_dock", action);
-
     action = new KAction(i18n("Focus Editor"), this);
     action->setShortcuts(QList<QKeySequence>() << (Qt::META | Qt::CTRL | Qt::Key_E) << Qt::META + Qt::Key_C);
     connect(action, SIGNAL(triggered(bool)), this, SLOT(focusEditor()));
@@ -98,88 +93,123 @@ MainWindowPrivate::MainWindowPrivate(MainWindow *w, Controller* controller)
     connect(action, SIGNAL(triggered(bool)), SLOT(selectPreviousDock()));
     ac->addAction("select_previous_dock", action);
 
-    action = new KAction(i18n("Remove view"), this);
-    connect(action, SIGNAL(triggered(bool)), SLOT(removeView()));
-    ac->addAction("remove_view", action);
-
-    action = new KAction(i18n("Anchor Current Dock"), this);
-    action->setCheckable(true);
-    action->setEnabled(false);
-    action->setShortcut(Qt::META | Qt::CTRL | Qt::Key_A);
-    connect(action, SIGNAL(toggled(bool)), SLOT(anchorCurrentDock(bool)));
-    ac->addAction("anchor_current_dock", action);
-
-    action = new KAction(i18n("Maximize Current Dock"), this);
-    action->setCheckable(true);
-    action->setEnabled(false);
-    connect(action, SIGNAL(toggled(bool)), SLOT(maximizeCurrentDock(bool)));
-    ac->addAction("maximize_current_dock", action);
-
     action = new KActionMenu(i18n("Tool Views"), this);
     ac->addAction("docks_submenu", action);
 
-    recreateCentralWidget();
+    idealController = new IdealController(m_mainWindow);
+
+    QToolBar *leftToolBar = new QToolBar(i18n("Left Button Bar"), m_mainWindow);
+    leftToolBar->setObjectName("left_button_bar");
+    leftToolBar->setMovable(false);
+    leftToolBar->setFloatable(false);
+    leftToolBar->layout()->setMargin(0);
+    leftToolBar->addWidget(idealController->leftBarWidget);
+    idealController->leftBarWidget->show();
+    m_mainWindow->addToolBar(Qt::LeftToolBarArea, leftToolBar);
+
+    QToolBar *rightToolBar = new QToolBar(i18n("Right Button Bar"), m_mainWindow);
+    rightToolBar->setObjectName("right_button_bar");
+    rightToolBar->setMovable(false);
+    rightToolBar->setFloatable(false);
+    rightToolBar->layout()->setMargin(0);
+    rightToolBar->addWidget(idealController->rightBarWidget);
+    idealController->rightBarWidget->show();
+    m_mainWindow->addToolBar(Qt::RightToolBarArea, rightToolBar);
+
+    QToolBar *bottomToolBar = new QToolBar(i18n("Bottom Button Bar"), m_mainWindow);
+    bottomToolBar->setObjectName("bottom_button_bar");
+    bottomToolBar->setMovable(false);
+    bottomToolBar->setFloatable(false);
+    bottomToolBar->layout()->setMargin(0);
+    bottomToolBar->addWidget(idealController->bottomBarWidget);
+    idealController->bottomBarWidget->show();
+    m_mainWindow->addToolBar(Qt::BottomToolBarArea, bottomToolBar);
+
+    // adymo: intentionally do not add a toolbar for top buttonbar
+    // this doesn't work well with toolbars added via xmlgui
+
+    centralWidget = new QWidget;
+    QVBoxLayout* layout = new QVBoxLayout(centralWidget);
+    centralWidget->setLayout(layout);
+    layout->setMargin(0);
+    splitterCentralWidget = new QSplitter(centralWidget);
+    layout->addWidget(splitterCentralWidget);
+    m_mainWindow->setCentralWidget(centralWidget);
+
+    connect(idealController,
+            SIGNAL(dockShown(Sublime::View*,Sublime::Position,bool)),
+            this,
+            SLOT(slotDockShown(Sublime::View*,Sublime::Position,bool)));
+
+    connect(idealController,
+            SIGNAL(widgetResized(Qt::DockWidgetArea,int)),
+            this,
+            SLOT(widgetResized(Qt::DockWidgetArea,int)));
+
+   connect(idealController, SIGNAL(dockBarContextMenuRequested(Qt::DockWidgetArea,QPoint)),
+            m_mainWindow, SLOT(dockBarContextMenuRequested(Qt::DockWidgetArea,QPoint)));
 }
 
 
 MainWindowPrivate::~MainWindowPrivate()
 {
-    delete m_leftTabbarCornerWidget;
+    delete m_leftTabbarCornerWidget.data();
+    m_leftTabbarCornerWidget.clear();
 }
 
 void MainWindowPrivate::showLeftDock(bool b)
 {
-    idealMainWidget->showLeftDock(b);
+    idealController->showLeftDock(b);
 }
 
 void MainWindowPrivate::showBottomDock(bool b)
 {
-    idealMainWidget->showBottomDock(b);
+    idealController->showBottomDock(b);
 }
 
 void MainWindowPrivate::showRightDock(bool b)
 {
-    idealMainWidget->showRightDock(b);
+    idealController->showRightDock(b);
 }
 
-void MainWindowPrivate::showTopDock(bool b)
+void MainWindowPrivate::setBackgroundCentralWidget(QWidget* w)
 {
-    idealMainWidget->showTopDock(b);
+    delete bgCentralWidget;
+    QLayout* l=m_mainWindow->centralWidget()->layout();
+    l->addWidget(w);
+    bgCentralWidget=w;
+    setBackgroundVisible(area->views().isEmpty());
 }
 
-void MainWindowPrivate::anchorCurrentDock(bool b)
+void MainWindowPrivate::setBackgroundVisible(bool v)
 {
-    idealMainWidget->anchorCurrentDock(b);
-}
-
-void MainWindowPrivate::maximizeCurrentDock(bool b)
-{
-    idealMainWidget->maximizeCurrentDock(b);
+    if(!bgCentralWidget)
+        return;
+    
+    bgCentralWidget->setVisible(v);
+    splitterCentralWidget->setVisible(!v);
 }
 
 void MainWindowPrivate::focusEditor()
 {
-    idealMainWidget->focusEditor();
+    if (View* view = m_mainWindow->activeView())
+        if (view->hasWidget())
+            view->widget()->setFocus(Qt::ShortcutFocusReason);
 }
 
 void MainWindowPrivate::toggleDocksShown()
 {
-    idealMainWidget->toggleDocksShown();
-}
-
-void MainWindowPrivate::removeView()
-{
-    idealMainWidget->removeView();
+    idealController->toggleDocksShown();
 }
 
 void MainWindowPrivate::selectNextDock()
 {
-    idealMainWidget->selectNextDock();
+    idealController->goPrevNextDock(IdealController::NextDock);
 }
 
 void MainWindowPrivate::selectPreviousDock()
 {
-    idealMainWidget->selectPreviousDock();
+    idealController->goPrevNextDock(IdealController::PrevDock);
 }
 
 Area::WalkerMode MainWindowPrivate::IdealToolViewCreator::operator() (View *view, Sublime::Position position)
@@ -187,14 +217,15 @@ Area::WalkerMode MainWindowPrivate::IdealToolViewCreator::operator() (View *view
     if (!d->docks.contains(view))
     {
         d->docks << view;
-        d->idealMainWidget->addView(d->positionToDockArea(position), view);
+
+        //add view
+        d->idealController->addView(d->positionToDockArea(position), view);
     }
     return Area::ContinueWalker;
 }
 
 Area::WalkerMode MainWindowPrivate::ViewCreator::operator() (AreaIndex *index)
 {
-    kDebug() << "reconstructing views for area index" << index;
     QSplitter *splitter = d->m_indexSplitters.value(index);
     if (!splitter)
     {
@@ -203,7 +234,7 @@ Area::WalkerMode MainWindowPrivate::ViewCreator::operator() (AreaIndex *index)
         {
             kDebug() << "reconstructing root area";
             //this is root area
-            splitter = new QSplitter(d->centralWidget);
+            splitter = d->splitterCentralWidget;
             d->m_indexSplitters[index] = splitter;
             d->centralWidget->layout()->addWidget(splitter);
         }
@@ -215,20 +246,34 @@ Area::WalkerMode MainWindowPrivate::ViewCreator::operator() (AreaIndex *index)
                 operator()(index->parent());
             }
             QSplitter *parent = d->m_indexSplitters.value(index->parent());
-            kDebug() << "adding new splitter to" << parent;
             splitter = new QSplitter(parent);
             d->m_indexSplitters[index] = splitter;
-            parent->addWidget(splitter);
+            
+            if(index == index->parent()->first())
+                parent->insertWidget(0, splitter);
+            else
+                parent->addWidget(splitter);
         }
         Q_ASSERT(splitter);
     }
-    splitter->show();
 
     if (index->isSplitted()) //this is a visible splitter
         splitter->setOrientation(index->orientation());
     else
     {
         Container *container = 0;
+        
+        if(splitter->count())
+        {
+            // After unsplitting, we might have to remove an old QSplitter here
+            QWidget* widget = splitter->widget(0);
+            if(qobject_cast<QSplitter*>(widget))
+            {
+                widget->setParent(0);
+                delete widget;
+            }
+        }
+        
         if (!splitter->widget(0))
         {
             //we need to create view container
@@ -236,8 +281,8 @@ Area::WalkerMode MainWindowPrivate::ViewCreator::operator() (AreaIndex *index)
             connect(container, SIGNAL(activateView(Sublime::View*)), d->m_mainWindow, SLOT(activateView(Sublime::View*)));
             connect(container, SIGNAL(tabContextMenuRequested(Sublime::View*,KMenu*)),
                     d->m_mainWindow, SLOT(tabContextMenuRequested(Sublime::View*,KMenu*)));
-            connect(container, SIGNAL(tabToolTipRequested(Sublime::View*,QPoint)),
-                    d->m_mainWindow, SLOT(tabToolTipRequested(Sublime::View*,QPoint)));
+            connect(container, SIGNAL(tabToolTipRequested(Sublime::View*,Sublime::Container*,int)),
+                    d->m_mainWindow, SLOT(tabToolTipRequested(Sublime::View*,Sublime::Container*,int)));
             connect(container, SIGNAL(closeRequest(QWidget*)),
                     d, SLOT(widgetCloseRequest(QWidget*)), Qt::QueuedConnection);
             splitter->addWidget(container);
@@ -250,14 +295,16 @@ Area::WalkerMode MainWindowPrivate::ViewCreator::operator() (AreaIndex *index)
         foreach (View *view, index->views())
         {
             QWidget *widget = view->widget(container);
-            if (widget && !container->hasWidget(widget))
+            if (widget)
             {
-                widget->installEventFilter(d);
-                foreach (QWidget* w, widget->findChildren<QWidget*>())
-                    w->installEventFilter(d);
-                container->addWidget(view, position);
-                d->viewContainers[view] = container;
-                d->widgetToView[widget] = view;
+                if(!container->hasWidget(widget))
+                {
+                    container->addWidget(view, position);
+                    d->viewContainers[view] = container;
+                    d->widgetToView[widget] = view;
+                }
+                if(d->activeView == view)
+                    container->setCurrentWidget(widget);
             }
             position++;
         }
@@ -265,53 +312,58 @@ Area::WalkerMode MainWindowPrivate::ViewCreator::operator() (AreaIndex *index)
     return Area::ContinueWalker;
 }
 
+void MainWindowPrivate::reconstructViews()
+{
+    ViewCreator viewCreator(this);
+    area->walkViews(viewCreator, area->rootIndex());
+    setBackgroundVisible(area->views().isEmpty());
+}
+
 void MainWindowPrivate::reconstruct()
 {
     if(m_leftTabbarCornerWidget) {
-        m_leftTabbarCornerWidget->hide();
-        m_leftTabbarCornerWidget->setParent(0);
+        m_leftTabbarCornerWidget.data()->hide();
+        m_leftTabbarCornerWidget.data()->setParent(0);
     }
-    
+
+    idealController->setWidthForArea(Qt::LeftDockWidgetArea, area->thickness(Sublime::Left));
+    idealController->setWidthForArea(Qt::BottomDockWidgetArea, area->thickness(Sublime::Bottom));
+    idealController->setWidthForArea(Qt::RightDockWidgetArea, area->thickness(Sublime::Right));
+
     IdealToolViewCreator toolViewCreator(this);
     area->walkToolViews(toolViewCreator, Sublime::AllPositions);
 
-    ViewCreator viewCreator(this);
-    area->walkViews(viewCreator, area->rootIndex());
+    reconstructViews();
 
-    idealMainWidget->blockSignals(true);
-    IdealMainLayout *l = idealMainWidget->mainLayout();
-    l->setWidthForRole(IdealMainLayout::Left, area->thickness(Sublime::Left));
-    l->setWidthForRole(IdealMainLayout::Right, area->thickness(Sublime::Right));
-    l->setWidthForRole(IdealMainLayout::Bottom,
-                       area->thickness(Sublime::Bottom));
-    l->setWidthForRole(IdealMainLayout::Top, area->thickness(Sublime::Top));
-    kDebug() << "RECONSTRUCT" << area << "  " << area->shownToolView(Sublime::Left) << "\n";
+    m_mainWindow->blockSignals(true);
+
+    kDebug() << "RECONSTRUCT" << area << "  " << area->shownToolViews(Sublime::Left) << "\n";
     foreach (View *view, area->toolViews())
     {
         QString id = view->document()->documentSpecifier();
         if (!id.isEmpty())
         {
             Sublime::Position pos = area->toolViewPosition(view);
-            if (area->shownToolView(pos) == id)
-                idealMainWidget->raiseView(view);
+            if (area->shownToolViews(pos).contains(id))
+                idealController->raiseView(view, IdealController::GroupWithOtherViews);
         }
     }
-    idealMainWidget->blockSignals(false);
+    m_mainWindow->blockSignals(false);
     
-    setTabBarLeftCornerWidget(m_leftTabbarCornerWidget);
+    setTabBarLeftCornerWidget(m_leftTabbarCornerWidget.data());
 }
 
 void MainWindowPrivate::clearArea()
 {
     if(m_leftTabbarCornerWidget)
-        m_leftTabbarCornerWidget->setParent(0);
+        m_leftTabbarCornerWidget.data()->setParent(0);
     
     //reparent toolview widgets to 0 to prevent their deletion together with dockwidgets
     foreach (View *view, area->toolViews())
     {
         // FIXME should we really delete here??
         bool nonDestructive = true;
-        idealMainWidget->removeView(view, nonDestructive);
+        idealController->removeView(view, nonDestructive);
 
         if (view->hasWidget())
             view->widget()->setParent(0);
@@ -326,52 +378,48 @@ void MainWindowPrivate::clearArea()
         if (view->hasWidget())
             view->widget()->setParent(0);
     }
-    recreateCentralWidget();
+    cleanCentralWidget();
     m_mainWindow->setActiveView(0);
     m_indexSplitters.clear();
     area = 0;
     viewContainers.clear();
     
-    setTabBarLeftCornerWidget(m_leftTabbarCornerWidget);
+    setTabBarLeftCornerWidget(m_leftTabbarCornerWidget.data());
 }
 
-void MainWindowPrivate::recreateCentralWidget()
+void MainWindowPrivate::cleanCentralWidget()
 {
-    idealMainWidget = new IdealMainWidget(m_mainWindow, m_mainWindow->actionCollection());
-    m_mainWindow->setCentralWidget(idealMainWidget);
+    while(splitterCentralWidget->count())
+        delete splitterCentralWidget->widget(0);
 
-    centralWidget = new QWidget();
-    idealMainWidget->setCentralWidget(centralWidget);
-
-    QVBoxLayout* layout = new QVBoxLayout(centralWidget);
-    layout->setMargin(0);
-    centralWidget->setLayout(layout);
-
-    connect(idealMainWidget,
-            SIGNAL(dockShown(Sublime::View*, Sublime::Position, bool)),
-            this,
-            SLOT(slotDockShown(Sublime::View*, Sublime::Position, bool)));
-
-    connect(idealMainWidget->mainLayout(),
-            SIGNAL(widgetResized(IdealMainLayout::Role, int)),
-            this,
-            SLOT(widgetResized(IdealMainLayout::Role, int)));
-
-    connect(idealMainWidget, SIGNAL(dockBarContextMenuRequested(Qt::DockWidgetArea, const QPoint&)),
-            m_mainWindow, SLOT(dockBarContextMenuRequested(Qt::DockWidgetArea, const QPoint&)));
+    setBackgroundVisible(true);
 }
 
+struct ShownToolViewFinder {
+    ShownToolViewFinder() {}
+    Area::WalkerMode operator()(View *v, Sublime::Position /*position*/)
+    {
+        if (v->hasWidget() && v->widget()->isVisible())
+            views << v;
+        return Area::ContinueWalker;
+    }
+    QList<View *> views;
+};
+
 void MainWindowPrivate::
 slotDockShown(Sublime::View* view, Sublime::Position pos, bool shown)
 {
     if (ignoreDockShown)
         return;
 
-    QString id;
-    if (shown)
-        id = view->document()->documentSpecifier();
-    kDebug() << "View " << view->document()->documentSpecifier() << " " << shown;
-    area->setShownToolView(pos, id);
+    ShownToolViewFinder finder;
+    m_mainWindow->area()->walkToolViews(finder, pos);
+
+    QStringList ids;
+    foreach (View *v, finder.views) {
+        ids << v->document()->documentSpecifier();
+    }
+    area->setShownToolViews(pos, ids);
 }
 
 void MainWindowPrivate::viewRemovedInternal(AreaIndex* index, View* view)
@@ -381,53 +429,74 @@ void MainWindowPrivate::viewRemovedInternal(AreaIndex* index, View* view)
     // A formerly non-empty working-set has become empty, and a relayout of the area-selector may be required
     if(m_mainWindow->area()->views().size() == 0)
         m_mainWindow->setupAreaSelector();
+    
+    setBackgroundVisible(area->views().isEmpty());
 }
 
 void MainWindowPrivate::viewAdded(Sublime::AreaIndex *index, Sublime::View *view)
 {
     if(m_leftTabbarCornerWidget) {
-        m_leftTabbarCornerWidget->hide();
-        m_leftTabbarCornerWidget->setParent(0);
+        m_leftTabbarCornerWidget.data()->hide();
+        m_leftTabbarCornerWidget.data()->setParent(0);
     }
-    
-    ViewCreator viewCreator(this);
-    QSplitter *splitter = m_indexSplitters[index];
-    if (index->isSplitted() && (splitter->count() == 1) &&
-            qobject_cast<Sublime::Container*>(splitter->widget(0)))
+
     {
-        Container *container = qobject_cast<Sublime::Container*>(splitter->widget(0));
-        //we need to remove extra container before reconstruction
-        //first reparent widgets in container so that they are not deleted
-        while (container->count())
+         // Remove container objects in the hierarchy from the parents,
+         // because they are not needed anymore, and might lead to broken splitter hierarchy and crashes.
+        for(Sublime::AreaIndex* current = index; current; current = current->parent())
         {
-            container->widget(0)->setParent(0);
+        QSplitter *splitter = m_indexSplitters[current];
+        if (current->isSplitted() && splitter)
+        {
+            // Also update the orientation
+            splitter->setOrientation(current->orientation());
+            
+            for(int w = 0; w < splitter->count(); ++w)
+            {
+                Container *container = qobject_cast<Sublime::Container*>(splitter->widget(w));
+                //we need to remove extra container before reconstruction
+                //first reparent widgets in container so that they are not deleted
+                if(container)
+                {
+                    while (container->count())
+                    {
+                        container->widget(0)->setParent(0);
+                    }
+                    //and then delete the container
+                    delete container;
+                }
+            }
+        }
         }
-        //and then delete the container
-        delete container;
     }
+
+    ViewCreator viewCreator(this);
     area->walkViews(viewCreator, index);
     emit m_mainWindow->viewAdded( view );
     
-    setTabBarLeftCornerWidget(m_leftTabbarCornerWidget);
+    setTabBarLeftCornerWidget(m_leftTabbarCornerWidget.data());
     
     // A formerly empty working-set may become non-empty, and a relayout of the area-selector may be required
     if(m_mainWindow->area()->views().size() == 1)
         m_mainWindow->setupAreaSelector();
+    
+    setBackgroundVisible(false);
 }
 
 void Sublime::MainWindowPrivate::raiseToolView(Sublime::View * view)
 {
-    idealMainWidget->raiseView(view);
+    idealController->raiseView(view);
 }
 
 void MainWindowPrivate::aboutToRemoveView(Sublime::AreaIndex *index, Sublime::View *view)
 {
-    if (!m_indexSplitters.contains(index))
+    if (!m_indexSplitters[index])
         return;
 
     QSplitter *splitter = m_indexSplitters[index];
     kDebug() << "index " << index << " root " << area->rootIndex();
     kDebug() << "splitter " << splitter << " container " << splitter->widget(0);
+    kDebug() << "structure: " << index->print() << " whole structure: " << area->rootIndex()->print();
     //find the container for the view and remove the widget
     Container *container = qobject_cast<Container*>(splitter->widget(0));
     if (!container) {
@@ -457,8 +526,8 @@ void MainWindowPrivate::aboutToRemoveView(Sublime::AreaIndex *index, Sublime::Vi
     else
     {
         if(m_leftTabbarCornerWidget) {
-            m_leftTabbarCornerWidget->hide();
-            m_leftTabbarCornerWidget->setParent(0);
+            m_leftTabbarCornerWidget.data()->hide();
+            m_leftTabbarCornerWidget.data()->setParent(0);
         }
         
         // We've about to remove the last view of this container.  It will
@@ -499,29 +568,30 @@ void MainWindowPrivate::aboutToRemoveView(Sublime::AreaIndex *index, Sublime::Vi
             AreaIndex *sibling = parent->first() == index ? parent->second() : parent->first();
             QSplitter *siblingSplitter = m_indexSplitters[sibling];
 
-            parentSplitter->setUpdatesEnabled(false);
-            //save sizes and orientation of the sibling splitter
-            parentSplitter->setOrientation(siblingSplitter->orientation());
-            QList<int> sizes = siblingSplitter->sizes();
-
-            /* Parent has two children -- 'index' that we've deleted and
-               'sibling'.  We move all children of 'sibling' into parent,
-               and delete 'sibling'.  sibling either contains a single
-               Container instance, or a bunch of further QSplitters.  */
-            while (siblingSplitter->count() > 0)
+            if(siblingSplitter)
             {
-                //reparent contents into parent splitter
-                QWidget *siblingWidget = siblingSplitter->widget(0);
-                siblingWidget->setParent(parentSplitter);
-                parentSplitter->addWidget(siblingWidget);
+                HoldUpdates du(parentSplitter);
+                //save sizes and orientation of the sibling splitter
+                parentSplitter->setOrientation(siblingSplitter->orientation());
+                QList<int> sizes = siblingSplitter->sizes();
+
+                /* Parent has two children -- 'index' that we've deleted and
+                'sibling'.  We move all children of 'sibling' into parent,
+                and delete 'sibling'.  sibling either contains a single
+                Container instance, or a bunch of further QSplitters.  */
+                while (siblingSplitter->count() > 0)
+                {
+                    //reparent contents into parent splitter
+                    QWidget *siblingWidget = siblingSplitter->widget(0);
+                    siblingWidget->setParent(parentSplitter);
+                    parentSplitter->addWidget(siblingWidget);
+                }
+
+                m_indexSplitters.remove(sibling);
+                delete siblingSplitter;
+                parentSplitter->setSizes(sizes);
             }
 
-            m_indexSplitters.remove(sibling);
-            delete siblingSplitter;
-
-            parentSplitter->setSizes(sizes);
-            parentSplitter->setUpdatesEnabled(true);
-
             kDebug() << "after deleation " << parent << " has "
                          << parentSplitter->count() << " elements";
 
@@ -531,13 +601,13 @@ void MainWindowPrivate::aboutToRemoveView(Sublime::AreaIndex *index, Sublime::Vi
             //activate the current view there
             if (containerToActivate) {
                 m_mainWindow->setActiveView(containerToActivate->viewForWidget(containerToActivate->currentWidget()));
-                setTabBarLeftCornerWidget(m_leftTabbarCornerWidget);
+                setTabBarLeftCornerWidget(m_leftTabbarCornerWidget.data());
                 return;
             }
         }
     }
 
-    setTabBarLeftCornerWidget(m_leftTabbarCornerWidget);
+    setTabBarLeftCornerWidget(m_leftTabbarCornerWidget.data());
     if ( wasActive ) {
         m_mainWindow->setActiveView(0L);
     }
@@ -554,7 +624,7 @@ void MainWindowPrivate::aboutToRemoveToolView(Sublime::View *toolView, Sublime::
     if (!docks.contains(toolView))
         return;
 
-    idealMainWidget->removeView(toolView);
+    idealController->removeView(toolView);
     // TODO are Views unique?
     docks.removeAll(toolView);
 }
@@ -565,7 +635,7 @@ void MainWindowPrivate::toolViewMoved(
     if (!docks.contains(toolView))
         return;
 
-    idealMainWidget->moveView(toolView, positionToDockArea(position));
+    idealController->moveView(toolView, positionToDockArea(position));
 }
 
 Qt::DockWidgetArea MainWindowPrivate::positionToDockArea(Position position)
@@ -598,17 +668,9 @@ void MainWindowPrivate::activateFirstVisibleView()
         m_mainWindow->activateView(area->views().first());
 }
 
-bool MainWindowPrivate::eventFilter(QObject *, QEvent *event)
+void MainWindowPrivate::widgetResized(Qt::DockWidgetArea dockArea, int thickness)
 {
-    if (event->type() == QEvent::FocusIn)
-        idealMainWidget->centralWidgetFocused();
-
-    return false;
-}
-
-void MainWindowPrivate::widgetResized(IdealMainLayout::Role role, int thickness)
-{
-    area->setThickness(IdealMainLayout::positionForRole(role), thickness);
+    //TODO: adymo: remove all thickness business
 }
 
 void MainWindowPrivate::widgetCloseRequest(QWidget* widget)
@@ -786,8 +848,10 @@ AreaTabButton::AreaTabButton ( QString text, QIcon icon, uint iconSize, QWidget*
 
 void MainWindowPrivate::setTabBarLeftCornerWidget(QWidget* widget)
 {
-    if(widget != m_leftTabbarCornerWidget)
-        delete m_leftTabbarCornerWidget;
+    if(widget != m_leftTabbarCornerWidget.data()) {
+        delete m_leftTabbarCornerWidget.data();
+        m_leftTabbarCornerWidget.clear();
+    }
     m_leftTabbarCornerWidget = widget;
     
     if(!widget || !area || viewContainers.isEmpty())
