@@ -17,12 +17,21 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
+#include "main.h"
+
 #include <shell/core.h>
 #include <shell/shellextension.h>
+
 #include <language/backgroundparser/parsejob.h>
+#include <language/backgroundparser/backgroundparser.h>
+#include <language/duchain/duchain.h>
+#include <language/duchain/duchainlock.h>
+
 #include <interfaces/ilanguage.h>
 #include <interfaces/iplugincontroller.h>
 #include <interfaces/ilanguagecontroller.h>
+#include <tests/autotestshell.h>
+#include <tests/testcore.h>
 
 #include <KDE/KApplication>
 #include <KDE/KCmdLineArgs>
@@ -32,12 +41,9 @@
 #include <QtCore/QStringList>
 #include <QtCore/QFile>
 #include <QtCore/QTimer>
+#include <QtCore/QDirIterator>
+
 #include <stdio.h>
-#include <tests/autotestshell.h>
-#include <QDirIterator>
-#include <language/backgroundparser/backgroundparser.h>
-#include "main.h"
-#include <language/duchain/duchain.h>
 
 bool verbose=false, warnings=false;
 
@@ -78,13 +84,10 @@ void Manager::init()
         QCoreApplication::exit(1);
     }
 
-    verbose=m_args->isSet("verbose");
-    warnings=m_args->isSet("warnings");
-
     uint features = TopDUContext::VisibleDeclarationsAndContexts;
-    if(m_args->isSet("f"))
+    if(m_args->isSet("features"))
     {
-        QString featuresStr = m_args->getOption("f");
+        QString featuresStr = m_args->getOption("features");
         if(featuresStr == "visible-declarations")
         {
             features = TopDUContext::VisibleDeclarationsAndContexts;
@@ -139,6 +142,8 @@ void Manager::init()
 
     if ( m_total ) {
         std::cout << "Added " << m_total << " files to the background parser" << std::endl;
+        const int threads = ICore::self()->languageController()->backgroundParser()->threadCount();
+        std::cout << "parsing with " << threads << " threads" << std::endl;
         if (m_waiting.isEmpty()) {
             std::cout << "ready" << std::endl;
             QApplication::quit();
@@ -156,7 +161,17 @@ void Manager::updateReady(IndexedString url, ReferencedTopDUContext topContext)
     m_waiting.remove(url.toUrl());
     
     std::cout << "processed " << (m_total - m_waiting.size()) << " out of " << m_total << std::endl;
-    
+    if (m_args->isSet("dump-errors") && topContext) {
+        DUChainReadLocker lock;
+        if (!topContext->problems().isEmpty()) {
+            std::cout << topContext->problems().size() << " problems encountered in " << topContext->url().c_str() << std::endl;
+            foreach(const ProblemPointer& p, topContext->problems()) {
+                std::cout << "  " << qPrintable(p->description()) << "\n    range: "
+                        << "[(" << p->finalLocation().start.line << ", " << p->finalLocation().start.column << "),"
+                        << " (" << p->finalLocation().end.line << ", " << p->finalLocation().end.column << ")]" << std::endl;
+            }
+        }
+    }
     if(m_waiting.isEmpty() && m_allFilesAdded)
     {
         std::cout << "ready" << std::endl;
@@ -198,7 +213,6 @@ QSet< KUrl > Manager::waiting()
 using namespace KDevelop;
 int main(int argc, char** argv)
 {
-    qInstallMsgHandler(messageOutput);
     KAboutData aboutData( "duchainify", 0, ki18n( "duchainify" ),
                           "1", ki18n("Duchain builder application"), KAboutData::License_GPL,
                           ki18n( "(c) 2009 David Nolden" ), KLocalizedString(), "http://www.kdevelop.org" );
@@ -206,24 +220,33 @@ int main(int argc, char** argv)
     KCmdLineOptions options;
     options.add("+dir", ki18n("directory"));
     
-    options.add("warnings", ki18n("Show warnings"));
-    options.add("verbose", ki18n("Show warnings and debug output"));
-    options.add("force-update", ki18n("Enforce an update of the top-contexts corresponding to the given files"));
-    options.add("force-update-recursive", ki18n("Enforce an update of the top-contexts corresponding to the given files and all included files"));
-    options.add("threads <count>", ki18n("Number of threads to use"));
-    options.add("f <features>", ki18n("Features to build. Options: empty, simplified-visible-declarations, visible-declarations (default), all-declarations, all-declarations-and-uses, all-declarations-and-uses-and-AST"));
+    options.add("w").add("warnings", ki18n("Show warnings"));
+    options.add("V").add("verbose", ki18n("Show warnings and debug output"));
+    options.add("u").add("force-update", ki18n("Enforce an update of the top-contexts corresponding to the given files"));
+    options.add("r").add("force-update-recursive", ki18n("Enforce an update of the top-contexts corresponding to the given files and all included files"));
+    options.add("t").add("threads <count>", ki18n("Number of threads to use"));
+    options.add("f").add("features <features>", ki18n("Features to build. Options: empty, simplified-visible-declarations, visible-declarations (default), all-declarations, all-declarations-and-uses, all-declarations-and-uses-and-AST"));
+    options.add("d").add("dump-errors", ki18n("Print problems encountered during parsing"));
     KCmdLineArgs::addCmdLineOptions( options );
 
     KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 
+    verbose = args->isSet("verbose");
+    warnings = args->isSet("warnings");
+    qInstallMsgHandler(messageOutput);
+
     KApplication app(false);
 
     AutoTestShell::init();
-    Core::initialize(0, KDevelop::Core::NoUi);
+    TestCore::initialize(Core::NoUi);
     Manager manager(args);
 
     QTimer::singleShot(0, &manager, SLOT(init()));
-    return app.exec();
+    int ret = app.exec();
+
+    TestCore::shutdown();
+
+    return ret;
 }
 
 #include "main.moc"
