@@ -321,8 +321,7 @@ void fillWorkingDirectory(Repo & r)
 void VcsBlackBoxTest::initTestCase()
 {
     AutoTestShell::init();
-    TestCore* core = new TestCore();
-    core->initialize(Core::Default);
+    TestCore::initialize(Core::Default);
     IPluginController *pluginController = Core::self()->pluginController();
     QList<IPlugin*> plugins = pluginController ->allPluginsForExtension("org.kdevelop.IBasicVersionControl");
 
@@ -334,9 +333,6 @@ void VcsBlackBoxTest::initTestCase()
             kDebug() << "Component: \"" << p->componentData().componentName() << "\" Name: \"" << idistr->name() << '"';
             QVERIFY2(!idistr->name().isEmpty(), "IBasicVersionControl may not return an empty name");
 
-            if (idistr->name() == "Git") // Needs some fixing first
-                continue;
-
             m_primary.push_back(RepoPtr(new DRepo(idistr)));
 
             m_secondary.push_back(RepoPtr(new DRepo(idistr)));
@@ -386,6 +382,7 @@ void VcsBlackBoxTest::initTestCase()
 
 void VcsBlackBoxTest::cleanupTestCase()
 {
+    TestCore::shutdown();
 }
 
 void VcsBlackBoxTest::repoInit(DRepo& r, DRepo& s)
