@@ -1,6 +1,6 @@
-
 /***************************************************************************
  *   Copyright 2008 Andreas Pakulat <apaku@gmx.de>                         *
+ *   Copyright 2010 Aleix Pol Gonzalez <aleixpol@kde.org>                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -70,9 +70,14 @@
 #include <qlabel.h>
 #include <QMenu>
 #include "widgets/vcsdiffpatchsources.h"
+#include "widgets/flexibleaction.h"
 #include <interfaces/isession.h>
 #include "vcsevent.h"
 #include <KCompositeJob>
+#include <QClipboard>
+#include <QApplication>
+#include <ktexteditor/modificationinterface.h>
+#include <QTimer>
 
 namespace KDevelop
 {
@@ -87,31 +92,32 @@ struct VcsPluginHelper::VcsPluginHelperPrivate {
     KAction * updateAction;
     KAction * historyAction;
     KAction * annotationAction;
-    KAction * diffToHeadAction;
     KAction * diffToBaseAction;
     KAction * revertAction;
     KAction * diffForRevAction;
+    KAction * diffForRevGlobalAction;
+    QPointer<QTimer> modificationTimer;
     
     void createActions(QObject * parent) {
         commitAction = new KAction(KIcon("svn-commit"), i18n("Commit..."), parent);
         updateAction = new KAction(KIcon("svn-update"), i18n("Update"), parent);
         addAction = new KAction(KIcon("list-add"), i18n("Add"), parent);
-        diffToHeadAction = new KAction(i18n("Compare to Head..."), parent);
-        diffToBaseAction = new KAction(i18n("Compare to Base..."), parent);
+        diffToBaseAction = new KAction(KIcon("vcs_diff"), i18n("Show Differences..."), parent);
         revertAction = new KAction(KIcon("archive-remove"), i18n("Revert"), parent);
         historyAction = new KAction(KIcon("view-history"), i18n("History..."), parent);
         annotationAction = new KAction(KIcon("user-properties"), i18n("Annotation..."), parent);
         diffForRevAction = new KAction(KIcon("vcs_diff"), i18n("Show Diff..."), parent);
+        diffForRevGlobalAction = new KAction(KIcon("vcs_diff"), i18n("Show Diff (all files)..."), parent);
         
         connect(commitAction, SIGNAL(triggered()), parent, SLOT(commit()));
         connect(addAction, SIGNAL(triggered()), parent, SLOT(add()));
         connect(updateAction, SIGNAL(triggered()), parent, SLOT(update()));
-        connect(diffToHeadAction, SIGNAL(triggered()), parent, SLOT(diffToHead()));
         connect(diffToBaseAction, SIGNAL(triggered()), parent, SLOT(diffToBase()));
         connect(revertAction, SIGNAL(triggered()), parent, SLOT(revert()));
         connect(historyAction, SIGNAL(triggered()), parent, SLOT(history()));
         connect(annotationAction, SIGNAL(triggered()), parent, SLOT(annotation()));
         connect(diffForRevAction, SIGNAL(triggered()), parent, SLOT(diffForRev()));
+        connect(diffForRevGlobalAction, SIGNAL(triggered()), parent, SLOT(diffForRevGlobal()));
     }
     
     bool allLocalFiles(const KUrl::List& urls)
@@ -145,7 +151,6 @@ struct VcsPluginHelper::VcsPluginHelperPrivate {
         menu->addSeparator();
         menu->addAction(historyAction);
         menu->addAction(annotationAction);
-        menu->addAction(diffToHeadAction);
         menu->addAction(diffToBaseAction);
         
         addAction->setEnabled(!allVersioned);
@@ -153,7 +158,6 @@ struct VcsPluginHelper::VcsPluginHelperPrivate {
         const bool singleVersionedFile = ctxUrls.count() == 1 && allVersioned;
         historyAction->setEnabled(singleVersionedFile);
         annotationAction->setEnabled(singleVersionedFile && allLocalFiles(ctxUrls));
-        diffToHeadAction->setEnabled(singleVersionedFile);
         diffToBaseAction->setEnabled(singleVersionedFile);
         commitAction->setEnabled(singleVersionedFile);
         
@@ -236,21 +240,52 @@ QMenu* VcsPluginHelper::commonActions()
 
 void VcsPluginHelper::revert()
 {
-    EXECUTE_VCS_METHOD(revert);
-}
-
-void VcsPluginHelper::diffToHead()
-{
-    SINGLEURL_SETUP_VARS
-    ICore::self()->documentController()->saveAllDocuments();
-    KDevelop::VcsJob* job = iface->diff(url,
-                                        KDevelop::VcsRevision::createSpecialRevision(KDevelop::VcsRevision::Head),
-                                        KDevelop::VcsRevision::createSpecialRevision(KDevelop::VcsRevision::Working));
-
-    connect(job, SIGNAL(finished(KJob*)), this, SLOT(diffJobFinished(KJob*)));
+    VcsJob* job=d->vcs->revert(d->ctxUrls);
+    connect(job, SIGNAL(finished(KJob*)), SLOT(revertDone(KJob*)));
+    
+    foreach(const KUrl& url, d->ctxUrls) {
+        IDocument* doc=ICore::self()->documentController()->documentForUrl(url);
+        
+        if(doc) {
+            KTextEditor::ModificationInterface* modif=dynamic_cast<KTextEditor::ModificationInterface*>(doc->textDocument());
+            modif->setModifiedOnDiskWarning(false);
+            doc->textDocument()->setModified(false);
+        }
+    }
+    job->setProperty("urls", d->ctxUrls);
+    
     d->plugin->core()->runController()->registerJob(job);
 }
 
+void VcsPluginHelper::revertDone(KJob* job)
+{
+    d->modificationTimer = new QTimer;
+    d->modificationTimer->setInterval(100);
+    connect(d->modificationTimer, SIGNAL(timeout()), SLOT(delayedModificationWarningOn()));
+    
+    d->modificationTimer->setProperty("urls", job->property("urls"));
+    d->modificationTimer->start();
+}
+
+void VcsPluginHelper::delayedModificationWarningOn()
+{
+    KUrl::List urls = d->modificationTimer->property("urls").value<KUrl::List>();
+    
+    foreach(const KUrl& url, urls) {
+        IDocument* doc=ICore::self()->documentController()->documentForUrl(url);
+        
+        if(doc) {
+            doc->reload();
+            
+            KTextEditor::ModificationInterface* modif=dynamic_cast<KTextEditor::ModificationInterface*>(doc->textDocument());
+            modif->setModifiedOnDiskWarning(true);
+        }
+    }
+    
+    d->modificationTimer->deleteLater();
+}
+
+
 void VcsPluginHelper::diffJobFinished(KJob* job)
 {
     KDevelop::VcsJob* vcsjob = qobject_cast<KDevelop::VcsJob*>(job);
@@ -259,9 +294,9 @@ void VcsPluginHelper::diffJobFinished(KJob* job)
     if (vcsjob->status() == KDevelop::VcsJob::JobSucceeded) {
         KDevelop::VcsDiff d = vcsjob->fetchResults().value<KDevelop::VcsDiff>();
         if(d.isEmpty())
-            KMessageBox::error(ICore::self()->uiController()->activeMainWindow(),
-                                i18n("There are no differences."),
-                                i18n("VCS support"));
+            KMessageBox::information(ICore::self()->uiController()->activeMainWindow(),
+                                     i18n("There are no differences."),
+                                     i18n("VCS support"));
         else {
             VCSDiffPatchSource* patch=new VCSDiffPatchSource(d);
             showVcsDiff(patch);
@@ -275,12 +310,9 @@ void VcsPluginHelper::diffToBase()
 {
     SINGLEURL_SETUP_VARS
     ICore::self()->documentController()->saveAllDocuments();
-    KDevelop::VcsJob* job = iface->diff(url,
-                                        KDevelop::VcsRevision::createSpecialRevision(KDevelop::VcsRevision::Base),
-                                        KDevelop::VcsRevision::createSpecialRevision(KDevelop::VcsRevision::Working));
 
-    connect(job, SIGNAL(finished(KJob*)), this, SLOT(diffJobFinished(KJob*)));
-    ICore::self()->runController()->registerJob(job);
+    VCSDiffPatchSource* patch =new VCSDiffPatchSource(new VCSStandardDiffUpdater(iface, url));
+    showVcsDiff(patch);
 }
 
 void VcsPluginHelper::diffForRev()
@@ -299,15 +331,23 @@ void VcsPluginHelper::diffForRev()
     d->plugin->core()->runController()->registerJob(job);
 }
 
-void VcsPluginHelper::history()
+void VcsPluginHelper::diffForRevGlobal()
+{
+    for(int a = 0; a < d->ctxUrls.size(); ++a)
+    {
+        KUrl& url(d->ctxUrls[a]);
+        IProject* project = ICore::self()->projectController()->findProjectForUrl( url );
+        if( project )
+            url = project->folder();
+    }
+    diffForRev();
+}
+
+void VcsPluginHelper::history(const VcsRevision& rev)
 {
     SINGLEURL_SETUP_VARS
-    KDevelop::VcsJob *job = iface->log(url);
-    if( !job ) 
-    {
-        kWarning() << "Couldn't create log job for:" << url << "with iface:" << iface << dynamic_cast<KDevelop::IPlugin*>( iface );
-        return;
-    }
+    KDevelop::VcsJob *job = iface->log(url, rev, VcsRevision::createSpecialRevision( VcsRevision::Start ));
+    
     KDialog* dlg = new KDialog();
     dlg->setButtons(KDialog::Close);
     dlg->setCaption(i18n("%2 History (%1)", url.pathOrUrl(), iface->name()));
@@ -341,8 +381,8 @@ void VcsPluginHelper::annotation()
             annotateiface->setAnnotationModel(model);
             viewiface->setAnnotationBorderVisible(true);
             connect(doc->textDocument()->activeView(),
-                    SIGNAL(annotationContextMenuAboutToShow( KTextEditor::View*, QMenu*, int )),
-                    this, SLOT(annotationContextMenuAboutToShow( KTextEditor::View*, QMenu*, int )));
+                    SIGNAL(annotationContextMenuAboutToShow(KTextEditor::View*,QMenu*,int)),
+                    this, SLOT(annotationContextMenuAboutToShow(KTextEditor::View*,QMenu*,int)));
         } else {
             KMessageBox::error(0, i18n("Cannot display annotations, missing interface KTextEditor::AnnotationInterface for the editor."));
             delete job;
@@ -353,6 +393,30 @@ void VcsPluginHelper::annotation()
     }
 }
 
+class CopyFunction : public AbstractFunction
+{
+    public:
+        CopyFunction(const QString& tocopy)
+            : m_tocopy(tocopy) {}
+        
+        void operator()() { QApplication::clipboard()->setText(m_tocopy); }
+    private:
+        QString m_tocopy;
+};
+
+class HistoryFunction : public AbstractFunction
+{
+    public:
+        HistoryFunction(VcsPluginHelper* helper, const VcsRevision& rev)
+            : m_helper(helper), m_rev(rev) {}
+            
+            void operator()() { m_helper->history(m_rev); }
+        
+    private:
+        VcsPluginHelper* m_helper;
+        VcsRevision m_rev;
+};
+
 void VcsPluginHelper::annotationContextMenuAboutToShow( KTextEditor::View* view, QMenu* menu, int line )
 {
     KTextEditor::AnnotationInterface* annotateiface =
@@ -363,7 +427,12 @@ void VcsPluginHelper::annotationContextMenuAboutToShow( KTextEditor::View* view,
 
     VcsRevision rev = model->revisionForLine(line);
     d->diffForRevAction->setData(QVariant::fromValue(rev));
+    d->diffForRevGlobalAction->setData(QVariant::fromValue(rev));
+    menu->addSeparator();
     menu->addAction(d->diffForRevAction);
+    menu->addAction(d->diffForRevGlobalAction);
+    menu->addAction(new FlexibleAction(KIcon("edit-copy"), i18n("Copy Revision"), new CopyFunction(rev.revisionValue().toString()), menu));
+    menu->addAction(new FlexibleAction(KIcon("view-history"), i18n("Revision History..."), new HistoryFunction(this, rev), menu));
 }
 
 void VcsPluginHelper::update()
@@ -382,66 +451,27 @@ void VcsPluginHelper::commit()
     ICore::self()->documentController()->saveAllDocuments();
 
     KUrl url = d->ctxUrls.first();
-    QScopedPointer<VcsJob> statusJob(d->vcs->status(url));
-    QMap<KUrl, VcsStatusInfo::State> changes;
-    if( statusJob->exec() && statusJob->status() == VcsJob::JobSucceeded )
-    {
-        QVariant varlist = statusJob->fetchResults();
-
-        foreach( const QVariant &var, varlist.toList() )
-        {
-            VcsStatusInfo info = qVariantValue<KDevelop::VcsStatusInfo>( var );
-            
-            if(info.state()!=VcsStatusInfo::ItemUpToDate)
-                changes[info.url()] = info.state();
-        }
-    }
-    else
-        kDebug() << "Couldn't get status for urls: " << url;
     
-    QScopedPointer<VcsJob> diffJob(d->vcs->diff(url,
-                                        KDevelop::VcsRevision::createSpecialRevision(KDevelop::VcsRevision::Base),
-                                        KDevelop::VcsRevision::createSpecialRevision(KDevelop::VcsRevision::Working)));
-
-    VcsDiff diff;
-    bool correctDiff = diffJob->exec();
-    if(correctDiff)
-        diff = diffJob->fetchResults().value<VcsDiff>();
-    
-    if(!correctDiff)
-        KMessageBox::error(0, i18n("Could not create a patch for the current version."));
-
     // We start the commit UI no matter whether there is real differences, as it can also be used to commit untracked files
-    VCSCommitDiffPatchSource* patchSource = new VCSCommitDiffPatchSource(diff, changes, d->vcs, retrieveOldCommitMessages());
+    VCSCommitDiffPatchSource* patchSource = new VCSCommitDiffPatchSource(new VCSStandardDiffUpdater(d->vcs, url), url, d->vcs);
     
     bool ret = showVcsDiff(patchSource);
-    
-    Q_ASSERT(ret && "Make sure PatchReview plugin is installed correctly");
-    if(ret) {
-        connect(patchSource, SIGNAL(reviewFinished(QString,QList<KUrl>)), this, SLOT(executeCommit(QString,QList<KUrl>)));
-        connect(patchSource, SIGNAL(reviewCancelled(QString)), this, SLOT(commitReviewCancelled(QString)));
+
+    if(!ret) {
+        VcsCommitDialog *commitDialog = new VcsCommitDialog(patchSource);
+        commitDialog->setCommitCandidates(patchSource->infos());
+        commitDialog->exec();
     } else {
-        delete patchSource;
+        connect(patchSource, SIGNAL(reviewFinished(QString,QList<KUrl>)), this, SLOT(commitReviewed(QString)));
+        connect(patchSource, SIGNAL(reviewCancelled(QString)), this, SLOT(commitReviewed(QString)));
     }
 }
 
-void VcsPluginHelper::executeCommit(const QString& message, const QList<KUrl>& urls)
-{
-    VcsJob* job=d->vcs->commit(message, urls, KDevelop::IBasicVersionControl::NonRecursive);
-    d->plugin->core()->runController()->registerJob(job);
-    addOldCommitMessage(message);
-}
-
-void VcsPluginHelper::commitReviewCancelled(QString message)
+void VcsPluginHelper::commitReviewed(QString message)
 {
     addOldCommitMessage(message);
 }
 
-void VcsPluginHelper::cancelCommit(KDevelop::VcsCommitDialog* dlg)
-{
-    dlg->deleteLater();
-}
-
 QStringList retrieveOldCommitMessages()
 {
     KConfigGroup vcsGroup(ICore::self()->activeSession()->config(), "VCS");
